#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""CLI testing fixtures"""
from typing import Any, Dict, List
from unittest.mock import patch
import responses
from c8ylp.main import cli
from c8ylp.cli.core import CliLogger


def mock_cli():

    with patch("c8ylp.cli.core.start_ssh") as mock:
        yield cli


class FixtureCumulocityAPI:
    def __init__(self, base_url="https://example.c8y.io", tenant="t12345") -> None:
        self.base_url = base_url
        self.tenant = tenant
        pass

    def simulate_pre_authenticated(
        self,
        serial_number: str = "ext-device-01",
        external_type: str = "c8y_Serial",
        tenant: str = None,
        device_managed_object: Dict[str, Any] = None,
        roles: List[str] = None,
    ):
        roles = roles if roles is not None else ["ROLE_REMOTE_ACCESS_ADMIN"]
        self.simulate_loginoptions()
        self.simulate_current_user(roles=roles)
        self.simulate_external_identity(
            serial_number=serial_number, external_type=external_type
        )
        self.simulate_managed_object(fragments=device_managed_object)

    def simulate_loginoptions(
        self, tenant: str = None, roles=["ROLE_REMOTE_ACCESS_ADMIN"]
    ):
        responses.add(
            responses.GET,
            f"{self.base_url}/tenant/loginOptions",
            json={
                "loginOptions": [
                    {
                        "initRequest": f"example?tenantId={tenant or self.tenant}",
                        "type": "OAUTH2_INTERNAL",
                    }
                ]
            },
            status=200,
        )

    def simulate_external_identity(
        self,
        tenant: str = None,
        external_type="c8y_Serial",
        serial_number="ext-device-01",
        managed_object_id="12345",
        code=200,
    ):
        responses.add(
            responses.GET,
            f"{self.base_url}/identity/externalIds/{external_type}/{serial_number}",
            json={
                "managedObject": {
                    "id": managed_object_id,
                },
            },
            status=code,
        )

    def simulate_managed_object(
        self,
        id: str = "12345",
        name: str = "device01",
        fragments: Dict[str, Any] = None,
        code=200,
    ):
        if fragments is None:
            fragments = {
                "c8y_RemoteAccessList": [
                    {
                        "id": "remote-access-id",
                        "name": "Passthrough",
                        "protocol": "PASSTHROUGH",
                        "port": 2001,
                    },
                ],
            }
        responses.add(
            responses.GET,
            f"{self.base_url}/inventory/managedObjects/{id}",
            json={
                "id": id,
                "name": name,
                **fragments,
            },
            status=code,
        )

    def simulate_current_user(self, roles: List[str] = None):
        roles = roles if roles is not None else ["ROLE_REMOTE_ACCESS_ADMIN"]
        user = {
            "effectiveRoles": [],
        }
        for role in roles:
            user["effectiveRoles"].append({"id": role, "name": role})
        responses.add(
            responses.GET, f"{self.base_url}/user/currentUser", json=user, status=200
        )

    def simulate_login_oauth(self, status_codes: List[int]):
        for status in status_codes:
            responses.add(
                responses.POST,
                f"{self.base_url}/tenant/oauth?tenant_id={self.tenant}",
                json={},
                adding_headers=[
                    ("Set-Cookie", "authorization=dummy-token-xyz; path=/; secure"),
                    ("Set-Cookie", "XSRF-TOKEN=my-XSRF-token; path=/; secure"),
                ],
                status=status,
            )
            # responses.add(
            #     responses.POST,
            #     f"{self.base_url}/tenant/oauth?tenant_id={self.tenant}",
            #     status=401,
            # )

        pass


class LocalProxyLog:
    """Local Proxy Log fixture"""

    def print_output(self):
        """Print log output"""
        log_path = CliLogger.log_path()
        if log_path.is_file():
            print(f"--localproxy.log ({log_path})--\n" + log_path.read_text())



#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""plugin command"""

import re
import os
import subprocess
import logging
import platform
import shutil
from pathlib import Path
from typing import Any, Dict, List, Optional
import click

from c8ylp import __ROOT_DIR__
from .. import options
from .core import ExitCodes, ProxyContext


def plugin_folders() -> List[Path]:
    """Plugin folders

    Returns:
        List[Path]: List of paths to search for plugins in
    """
    paths = [
        Path(__ROOT_DIR__) / "plugins",
    ]

    # optional add custom plugins location
    user_path = os.getenv("C8YLP_PLUGINS", "~/.c8ylp/plugins")
    if user_path:
        for path in user_path.split(";"):
            if path:
                paths.append(Path(path))

    return paths


def load_python_plugin(path: str) -> Dict[Any, Any]:
    """Load a python plugin from a file

    Args:
        path (str): Path

    Returns:
        Dict[Any, Any]: Namespace of the evaluated python file
    """
    namespace = {}
    with open(path) as file:
        code = compile(file.read(), str(path), "exec")
        # Eval is required to make a dynamic plugin
        # pylint: disable=eval-used
        eval(code, namespace, namespace)
    return namespace


def format_wsl_path(path: str) -> str:
    """Format windows path to the WSL equivalent

    Args:
        path (str): Windows path to convert

    Returns:
        str: WSL path, i.e. /mnt/c/my/path.sh
    """
    out_path = path.replace("\\", "/", -1)

    if re.search("^[a-z]:/", out_path, re.IGNORECASE):
        drive, _, rest = out_path.partition(":")
        out_path = f"/mnt/{drive.lower()}{rest}"

    return out_path


def build_cmd_args(cmd_args: List[str]) -> List[str]:
    """Build the command arguments to launch the plugin

    It allows for setting a custom shell via C8YLP_SHELL environment
    variable.

    Windows has special handling due to WSL and shell scripts not being
    executable by default.

    Args:
        cmd_args (List[str]): List of command arguments (without the shell)

    Returns:
        List[str]: Command arguments to execute specific to the OS

    """
    custom_shell = os.getenv("C8YLP_SHELL")
    out_args = [*cmd_args]

    if platform.system() != "Windows":
        if not custom_shell:
            # Rely on the script's shebang
            return out_args

        if not shutil.which(custom_shell):
            raise Exception(f"Could not find {custom_shell}")

        return [shutil.which(custom_shell)] + out_args

    #
    # Windows is more tricky as it does not read the bash shebang
    # so we have to help it out, but also being WSL friendly
    #
    shell = custom_shell or "bash"
    if not shutil.which(shell):
        # Let windows take care about executing the
        # default app associated
        return ["cmd.exe", "/c"] + out_args

    shell = [shutil.which(shell)]

    if (
        "wsl".casefold() in shell[0].casefold()
        or "\\windows\\system32\\bash.exe".casefold() in shell[0].casefold()
    ):
        #
        # WSL is going to be used, therefore the path needs to be reformatted
        # otherwise the path will not be found due to windows/linux differences
        # i.e. "C:\my\script\plugin.sh" => "/mnt/c/script/plugin.sh"
        out_args[0] = format_wsl_path(out_args[0])

    return shell + out_args


def create_bash_plugin(script_path: str) -> click.Command:
    """Create bash plugin wrapped in a click.Command

    Args:
        script_path (str): Path to the bash script

    Returns:
        click.Command: Command
    """

    @click.command()
    @options.common_options
    @click.argument(
        "additional_args",
        metavar="[ARGS]...",
        nargs=-1,
        type=click.UNPROCESSED,
    )
    @click.pass_context
    def bash_wrapper(wctx, additional_args, **kwargs):
        proxy = ProxyContext(wctx, kwargs).start_background()
        proxy.set_env()

        plugin_args = build_cmd_args([script_path, *additional_args])
        proxy.show_message(f"Running plugin: {' '.join(plugin_args)}")
        try:
            exit_code = subprocess.call(plugin_args, env=os.environ)
        except Exception as ex:
            proxy.show_error(
                "Failed to execute plugin. Is the plugin executable and contain a shebang on the first line?. "
                f"plugin={script_path}, error={ex}"
            )
            exit_code = ExitCodes.PLUGIN_EXECUTION_ERROR
        wctx.exit(exit_code)

    return bash_wrapper


class PluginCLI(click.MultiCommand):
    """Plugin CLI Command which can automatically discover plugins at runtime"""

    def list_commands(self, ctx: click.Context) -> List[str]:
        """List commands

        Args:
            ctx (click.Context): Click context

        Returns:
            List[str]: List of sub commands
        """
        cmds = []

        for plugin_folder in plugin_folders():
            click.echo(f"Checking plugin folder: {plugin_folder}")
            plugin_folder = plugin_folder.expanduser()
            if plugin_folder.exists():
                for filename in os.listdir(str(plugin_folder)):
                    if filename.endswith(".py") and filename != "__init__.py":
                        cmds.append(filename[:-3])
                    elif filename.endswith(".sh"):
                        cmds.append(filename[:-3])

        cmds.sort()

        logging.info("Detected plugins: %s", cmds)
        return cmds

    def get_command(self, ctx: click.Context, cmd_name: str) -> Optional[click.Command]:
        """Get a command via its name. It will dynamically execute the
        plugin (if it is a python plugin) or wrap it in a click command
        if it is a bash plugin.

        When resilient parsing is activated (mostly during tab completion), then don't
        trigger an exit as it can affect the completion results from being displayed.

        Args:
            ctx (Context): Click context
            name (str): Name of the command

        Returns:
            Optional[click.Command]: Resolved command
        """
        # pylint: disable=too-many-branches
        namespace = {}

        file_exts = [".py", ".sh"]

        plugins = []
        for plugin_folder in plugin_folders():
            plugin_folder = plugin_folder.expanduser()

            for ext in file_exts:
                path = plugin_folder.joinpath(cmd_name + ext)
                if os.path.exists(path):
                    plugins.append(path)

        if not plugins:
            if ctx.resilient_parsing:
                return None
            logging.warning("Could not find plugin. name=%s", cmd_name)
            ctx.exit(ExitCodes.PLUGIN_NOT_FOUND)

        if cmd_name:
            logging.info("plugin found: name=%s, path=%s", cmd_name, plugins)

        for path in plugins:
            if path.suffix == ".py":
                logging.debug("Detected python script: %s", path)
                try:
                    namespace = load_python_plugin(str(path))
                except Exception as ex:
                    logging.warning("Failed to load python plugin. error=%s", ex)
                    if not ctx.resilient_parsing:
                        ctx.exit(ExitCodes.PLUGIN_INVALID_FORMAT)

                break

            if path.suffix == ".sh":
                namespace["cli"] = create_bash_plugin(str(path))
                break

        if "cli" not in namespace:
            if cmd_name != ctx.parent.command.name:
                logging.warning(
                    "Plugin is missing a function called 'cli'. %s, parent=%s",
                    cmd_name,
                    ctx.parent.command.name,
                )
                if ctx.resilient_parsing:
                    return None
                ctx.exit(ExitCodes.PLUGIN_INVALID_FORMAT)

        return namespace.get("cli")


@click.group()
@options.LOGGING_VERBOSE
def cli_plugin():
    """Plugins"""


@cli_plugin.command(
    "plugin",
    cls=PluginCLI,
    hidden=False,
    options_metavar="",
    context_settings=dict(
        ignore_unknown_options=True,
    ),
)
def run():
    """
    Run a custom plugin (installed under ~/.c8ylp/plugins/)

    Example 1:

    \b
        c8ylp plugin command device01 -- ssh -p \\$PORT myuser@localhost
    """



#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Custom request sessions"""

import logging
import requests


class BaseUrlSession(requests.Session):
    """Cumulocity requests session which automatically adds the given prefix
    to urls when sending the requests (if the url is not already fully defined)

    i.e. inventory/managedObjects => https://cumulocity.com/inventory/managedObjects

    References:
        * https://github.com/psf/requests/issues/2554
    """

    def __init__(self, prefix_url: str):
        if not prefix_url.startswith("http"):
            # default to https!
            prefix_url = f"https://{prefix_url}"
        self.prefix_url = prefix_url
        self.default_timeout = 30.0
        super().__init__()

    def expand_url(self, url: str) -> str:
        """Example the url if only a partial url (i.e /event/events) is given.

        Args:
            url (str): Partial or full url

        Returns:
            str: Returns full url (prepends the prefixUrl if a partial url is detected)
        """
        if not url.startswith("http"):
            url = self.prefix_url + "/" + url.lstrip("/")
        return url

    def request(self, method: str, url: str, *args, **kwargs) -> requests.Response:
        """Override default request method to automatically add the host to the url
        if the given url does not start with http

        Args:
            method (str): Request method (see requests.request for more details)
            url (str): Partial url (which will be joined with the prefix_url)
                       If it starts with http, then the prefix will not be prepended
            args: additional args passed to the requests method
            kwargs: additional args passed to the requests method

        Returns:
            requests.Response: Request response
        """

        # pylint: disable=arguments-differ
        url = self.expand_url(url)

        send_kwargs = {
            "timeout": kwargs.pop("timeout", self.default_timeout),
            "proxies": kwargs.pop("proxies", self.proxies),
            "allow_redirects": kwargs.pop("allow_redirects", True),
            "stream": kwargs.pop("stream", self.stream),
            "verify": kwargs.pop("verify", self.verify),
            "cert": kwargs.pop("cert", self.cert),
        }

        prepped = super().prepare_request(
            requests.Request(method, url, *args, **kwargs)
        )
        logging.debug("Sending requests to %s", url)
        response = super().send(prepped, **send_kwargs)
        return response



#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Test command timer"""
import time
import pytest
from c8ylp.timer import CommandTimer


def test_timer_calling_stop_before_start():
    """Test calling stop before timer has started"""
    timer = CommandTimer("My task")
    assert timer.stop() == 0


def test_timer_context(capfd: pytest.CaptureFixture):
    """Test timing using context"""
    timer = CommandTimer("My task", on_exit=print)

    with timer:
        time.sleep(1)

    # Use approx due to differences in precision of timers on different OS's
    assert pytest.approx(timer.last_duration, abs=0.5) == 1.0
    out, err = capfd.readouterr()
    assert out == "My task: 0:00:01\n"
    assert not err



#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""SSH command tests"""
import pytest
import responses
from click.testing import CliRunner
from c8ylp.main import cli
from tests.env import Environment
from tests.fixtures import FixtureCumulocityAPI


@pytest.mark.parametrize(
    "inputs",
    (
        {
            "stdin": [
                "https://example.c8y.io\n",
                "example-user\n",
                "dummy-password\n",
                "12345\n",
            ],
            "env": {},
        },
        {
            "stdin": [
                "https://example.c8y.io\n",
                "dummy-password\n",
                "12345\n",
            ],
            "env": {
                "C8Y_USER": "example-user",
            },
        },
        {
            "stdin": [
                "https://example.c8y.io\n",
                "12345\n",
            ],
            "env": {
                "C8Y_USER": "example-user",
                "C8Y_PASSWORD": "example-user",
            },
        },
        {
            "stdin": [
                "12345\n",
            ],
            "env": {
                "C8Y_USER": "example-user",
                "C8Y_PASSWORD": "example-user",
                "C8Y_HOST": "https://example.c8y.io",
            },
        },
    ),
)
def test_prompt_for_details(
    inputs, c8yserver: FixtureCumulocityAPI, env: Environment, tmpdir
):
    """Ask user for missing parameters"""

    @responses.activate
    def run():
        c8yserver.simulate_loginoptions()
        c8yserver.simulate_login_oauth(status_codes=[401, 200])

        env_file = tmpdir.join(".env")
        stdin = "".join(inputs["stdin"])

        runner = CliRunner()
        result = runner.invoke(
            cli,
            ["login", "--env-file", env_file.strpath],
            env={
                **env.create_empty_env(),
                **inputs["env"],
            },
            input=stdin,
        )

        assert result.exit_code == 0

        settings = {}
        for line in env_file.readlines():
            key, _, value = line.partition("=")
            settings[key] = value.rstrip("\n")

        assert settings == {
            "C8Y_HOST": "https://example.c8y.io",
            "C8Y_USER": "example-user",
            "C8Y_TENANT": "t12345",
            "C8Y_TOKEN": "dummy-token-xyz",
        }

    run()


def test_repeated_login_failures(c8yserver: FixtureCumulocityAPI, env: Environment):
    """Test repeated login failures. Eventually c8ylp should give up"""

    @responses.activate
    def run():
        c8yserver.simulate_loginoptions()
        c8yserver.simulate_login_oauth(status_codes=[401, 401, 401, 401])

        stdin = "".join(
            [
                "password\n",
                "tfa\n",
                "password\n",
                "tfa\n",
            ]
        )

        runner = CliRunner()
        result = runner.invoke(
            cli,
            ["login", "--host", c8yserver.base_url, "--user", "dummy_user"],
            env={
                **env.create_empty_env(),
            },
            input=stdin,
        )

        assert result.exit_code == 2

    run()


def test_disable_prompts(c8yserver: FixtureCumulocityAPI, env: Environment):
    """Test disabling of prompts for usage in scripts"""

    @responses.activate
    def run():
        c8yserver.simulate_loginoptions()
        c8yserver.simulate_login_oauth(status_codes=[401, 401, 401, 401])

        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "login",
                "--host",
                c8yserver.base_url,
                "--user",
                "dummy_user",
                "--disable-prompts",
            ],
            env={
                **env.create_empty_env(),
            },
        )

        assert result.exit_code == 2

    run()



#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Help tests"""
import re
import subprocess
import sys
from click.testing import CliRunner
from c8ylp.main import cli


def test_calling_root_module():
    """Test calling root module using python3 -m syntax"""

    # pylint: disable=subprocess-run-check
    proc = subprocess.run(
        [sys.executable, "-m", "c8ylp"],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    assert proc.returncode == 0

    output = proc.stdout.decode()
    assert re.search("Usage: python3? -m c8ylp ", output)
    assert "Options:" in output


def test_help_without_credentials():
    """Test calling help without any required arguments
    i.e. c8ylp --help
    """
    runner = CliRunner()
    result = runner.invoke(cli, ["--help"])
    assert "Usage: cli" in result.output
    assert "Options:" in result.output



#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""c8ylp version"""
import os
import importlib

__ROOT_DIR__ = os.path.dirname(os.path.abspath(__file__))  # This is your Project Root


def _get_version():
    # pylint: disable=bare-except, import-outside-toplevel
    try:
        # Needs to be dynamically imported as the version file is not
        # committed to the project as it is generated during the build.
        version = importlib.import_module("c8ylp._version")
        return version.version
    except:
        return "0.0.1"


__version__ = _get_version()



#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""server command"""

import click

from .. import options
from .core import ProxyContext


@click.command()
@options.common_options
@click.pass_context
def server(
    ctx,
    *_args,
    **kwargs,
):
    """Start local proxy in server mode

    \b
        DEVICE is the device's external identity

    Once the local proxy has started, clients such as ssh and scp can be used
    to establish a connection to the device.

    \b
    Example 1: Start the local proxy, prompt for credentials (if not set via env variables)

        \b
        c8ylp server device01 --host https://example.c8y.io

    Example 2: Start the local proxy using environment file

        \b
        c8ylp server --env-file .env device01

    Example 3: Start the local proxy with randomized port

        \b
        c8ylp server --env-file .env --port 0 device01
    """
    proxy = ProxyContext(ctx, kwargs)
    proxy.start()



#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Test environment"""

import os
from typing import Any, Dict
from c8ylp.env import loadenv


class Environment:
    def __init__(self, file=".env") -> None:
        if os.path.exists(file):
            loadenv(file)

        self.data = os.environ

    def env(self) -> Dict[str, str]:
        return self.data

    def create_empty_env(self) -> Dict[str, Any]:
        empty_env = {}
        for key in self.data:
            if key.startswith("C8Y"):
                empty_env[key] = None
        return empty_env

    def create_authenticated(self) -> Dict[str, Any]:
        return {
            "C8Y_HOST": "https://example.c8y.io",
            "C8Y_TENANT": "t12345",
            "C8Y_USER": "user01",
            "C8Y_TOKEN": "dummy-token",
        }

    def create_username_password(self) -> Dict[str, Any]:
        return {
            "C8Y_HOST": "https://example.c8y.io",
            "C8Y_TENANT": "t12345",
            "C8Y_USER": "user01",
            "C8Y_PASSWORD": "som4-p4$swurd",
        }


ENV = Environment()
